use twmap::*;
use twmap::convert::*;
use std::process::exit;
use std::ffi::OsStr;
use clap::{App, Arg};
use serde::Serialize;

#[derive(Default, Serialize)]
struct Output {
    starts: Vec<Point>,
    finishes: Vec<Point>,
}

#[derive(Serialize)]
struct Point {
    x: u32,
    y: u32,
}

fn main() {
    let matches = App::new("twmap start_finish_tiles")
        .about("Outputs coordinates of start and finish tiles, in the format y,x")
        .arg(Arg::with_name("file")
            .required(true)
            .index(1))
        .get_matches();
    match print_start_finish_tiles(matches.value_of_os("file").unwrap()) {
        Ok(output) => println!("{}", serde_json::to_string_pretty(&output).unwrap()),
        Err(err) => {
            eprintln!("{}", err);
            exit(1);
        }
    }
}

fn print_start_finish_tiles(path: &OsStr) -> Result<Output, Error> {
    let mut output = Output::default();
    let mut map = TwMap::parse_path(path)?;
    map.groups.load_conditionally(|layer| [LayerKind::Game, LayerKind::Front].contains(&layer.kind()))
        .map_err(|err| Error::Map(err.into()))?;
    let mut game_tiles = Vec::new();
    game_tiles.push(map.find_physics_layer::<GameLayer>().unwrap().tiles.unwrap_ref());
    if let Some(front_layer) = map.find_physics_layer::<FrontLayer>() {
        game_tiles.push(front_layer.tiles.unwrap_ref());
    }
    for tiles in game_tiles {
        tiles.indexed_iter()
            .for_each(|((y, x), tile)| {
                let (y, x) = (y.try_to(), x.try_to());
                match tile.id {
                    33 => output.starts.push(Point{ x, y }),
                    34 => output.finishes.push(Point { x, y }),
                    _ => {},
                }
            });
    }

    Ok(output)
}